!===============================================================
!
! Copyright (C) 2005 Finite Difference Research Group
! This file is part of parsec, http://www.ices.utexas.edu/parsec/
!
! Matrix-vector multiplication for the spin-orbit component of
! pseudopotentials.
!
!---------------------------------------------------------------
subroutine zmatvec_so(kplp,nloc_p_pot,u_pot,solver,parallel,vnew,p,q,m)

  use constants
  use matvec_module
  use non_local_psp_module
  use eigen_solver_module
  use parallel_data_module
  implicit none
  !
  ! Input/Output variables:
  !
  ! non local pseudopotential related data
  type (nonloc_pseudo_potential), intent(in) :: nloc_p_pot 
  ! U potential related data
  type (nonloc_pseudo_potential), intent(in) :: u_pot
  ! solver related data
  type (eigen_solver), intent(inout) :: solver
  ! parallel computation related data
  type (parallel_data), intent(in) :: parallel
  ! vnew
  real(dp), intent(in) :: vnew(parallel%mydim,2)
  ! number of vectors
  integer, intent(in) :: m
  ! index of k-point
  integer, intent(in) :: kplp
  ! input vectors in IW, distributed accross PEs
  complex(dpc), intent(in) :: p(2*parallel%ldn,m)
  ! output vectors in IW, distributed accross PEs
  complex(dpc), intent(out) :: q(2*parallel%ldn,m)
  !
  ! Work variables:
  complex(dpc), dimension(:,:), allocatable :: qup, qdn, pup, pdn
  !
  ! local scalars
  integer i, ldn, ldn2, dim, dim2
  !---------------------------------------------------------------
  allocate(qup(parallel%ldn,m))
  allocate(qdn(parallel%ldn,m))
  allocate(pup(parallel%ldn,m))
  allocate(pdn(parallel%ldn,m))
  q(:,:) = zzero
  qup(:,:) = zzero
  qdn(:,:) = zzero
  pup(:,:) = zzero
  pdn(:,:) = zzero
  ldn = parallel%ldn
  ldn2 = 2*parallel%ldn
  dim = parallel%mydim
  dim2 = 2*parallel%mydim

  pup(1:dim,1:m) = p(1:dim,1:m)
  pdn(1:dim,1:m) = p(1+dim:dim2,1:m)

  ! applying the scalar-relativistic Hamiltonian on the "up" part
  !------------------------------------------------------------------
  if (nloc_p_pot%nkpt == 0) then
     call dcopy(parallel%mydim,vnew(1,1),1,solver%adiag,1)
     solver%adiag(:) = solver%adiag(:) + sum(solver%coe2(0,:))
  else
     call dcopy(parallel%mydim,vnew(1,1),1,solver%adiag,1)
     solver%adiag(:) = solver%adiag(:) + sum(solver%coe2(0,:)) + &
          nloc_p_pot%kpts(1,kplp)*nloc_p_pot%kpts(1,kplp) + &
          nloc_p_pot%kpts(2,kplp)*nloc_p_pot%kpts(2,kplp) + &
          nloc_p_pot%kpts(3,kplp)*nloc_p_pot%kpts(3,kplp)
  endif

  call zmatvec_nloc(kplp, nloc_p_pot, u_pot, solver, parallel, pup, qup, m)
  if(nloc_p_pot%nkpt == 0) then
     call Zmatvec_ke(solver, parallel, pup, qup, m)
  else
     call Zmatvec_ke_kpt(kplp, solver, parallel, pup, qup, m)
  endif

  ! applying the scalar-relativistic Hamiltonian on the "dn" part
  !------------------------------------------------------------------
  if (nloc_p_pot%nkpt == 0) then
     call dcopy(parallel%mydim,vnew(1,2),1,solver%adiag,1)
     solver%adiag(:) = solver%adiag(:) + sum(solver%coe2(0,:))
  else
     call dcopy(parallel%mydim,vnew(1,2),1,solver%adiag,1)
     solver%adiag(:) = solver%adiag(:) + sum(solver%coe2(0,:)) + &
          nloc_p_pot%kpts(1,kplp)*nloc_p_pot%kpts(1,kplp) + &
          nloc_p_pot%kpts(2,kplp)*nloc_p_pot%kpts(2,kplp) + &
          nloc_p_pot%kpts(3,kplp)*nloc_p_pot%kpts(3,kplp)
  endif

  call Zmatvec_nloc(kplp, nloc_p_pot, u_pot, solver, parallel, pdn, qdn, m)
  if(nloc_p_pot%nkpt == 0) then
     call Zmatvec_ke(solver, parallel, pdn, qdn, m)
  else
     call Zmatvec_ke_kpt(kplp, solver, parallel, pdn, qdn, m)
  endif
  q(1:dim,1:m) = qup(1:dim,1:m)
  q(1+dim:dim2,1:m) = qdn(1:dim,1:m)
  ! applying the spin-orbit Hamiltonian on the "up" part
  !---------------------------------------------------------
  qup(:,:) = zzero
  qdn(:,:) = zzero
  do i = 1, m
     call LzSz(nloc_p_pot,parallel,pup(1,i),qup(1,i),dim,1,kplp)
     q(1:dim,i) = q(1:dim,i) + qup(1:dim,i)

     call LSxy(nloc_p_pot,parallel,pup(1,i),qdn(1,i),dim,1,kplp)
     q(1+dim:dim2,i) = q(1+dim:dim2,i) + qdn(1:dim,i)
  enddo

  ! applying the spin-orbit Hamiltonian on the "dn" part      
  !-----------------------------------------------------------
  qup(:,:) = zzero
  qdn(:,:) = zzero
  do i = 1, m
     call LzSz(nloc_p_pot,parallel,pdn(1,i),qdn(1,i),dim,-1,kplp)
     q(1+dim:dim2,i) = q(1+dim:dim2,i) + qdn(1:dim,i)
     
     call LSxy(nloc_p_pot,parallel,pdn(1,i),qup(1,i),dim,-1,kplp)
     q(1:dim,i) = q(1:dim,i) + qup(1:dim,i)
  enddo
  deallocate(qup, qdn, pup, pdn)

end subroutine zmatvec_so
!===============================================================
